// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*****************************************************************************
 *
 *  stringFns.cc                                XMM Science Analysis Subsystem
 *
 *  string functions
 *
 *  (c) 1999 ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: StringFns.cc,v 1.5 2008/09/26 16:38:55 eojero Exp $
 *
 *****************************************************************************/

#include "StringFns.h"       // Application headers
#include <cctype>

/*** Split a string into tokens, using specified separator character ***/

std::list<std::string> StringFns::splitString(const std::string& s, const std::string &separator)
{
    std::list<std::string> strlist;

    if(s.length() > 0)
    {
        std::string::size_type i = s.find_first_not_of(separator);
        std::string::size_type j = 0;
        while(j != std::string::npos && i != std::string::npos)
        {
            j = s.find_first_of(separator,i);
            strlist.push_back(s.substr(i,j-i));
            i = s.find_first_not_of(separator,j);
        }
    }
    return strlist;
}


/*
 *  Split a quoted string into tokens. Set 'ok=false' if quotes are mismatched.
 */

std::list<std::string> StringFns::splitStringList(const std::string& str, bool* ok)
{
    bool misMatch = false;
    std::list<std::string> strlist;
    const char* p = str.c_str();
    char c =*p++;

    while(c)
    {
        std::string token = "";

        // Skip leading space
	while(c && (c ==' ' || c == '\t')) 
	    c=*p++;

	// Quotes
	if(c == '"' || c == '\'') {
	    char quote = c;
	    c=*p++;
	    while(c && (c != quote)) {
	        token += c;
		c=*p++;
	    }
	    if(c != quote)
	        misMatch = true;           // Mis-matched quotes

	    c=*p++;
	    strlist.push_back(token);
	} 
	else if(c && (c != ' ' && c != '\t')) 
	{
	    // Simple token
	    while(c && (c != ' ' && c != '\t' && c != '"')) {
	        token += c;
		c=*p++;
	    }
	    strlist.push_back(token);
	}
    }
    if(ok)
        *ok = !misMatch;

    return strlist;
}


/*
 *  Split a string into tokens separated by delimiter characters.
 */

std::list<std::string> StringFns::splitDelimitedString(const std::string& s, const std::string& delim)
{
    std::list<std::string> strlist;

    if(s.length() > 0)
    {
        std::string::size_type i = 0;
        std::string::size_type j = 0;
        while(j != std::string::npos && i != std::string::npos)
        {
            j = s.find_first_of(delim,i);
            strlist.push_back(s.substr(i,j-i));
            i = j+1; 
        }
    }
    return strlist;
}


/*
 *  Parse an integer.
 *  If valid, return true and pass result via argument 'result'.
 */

bool StringFns::parseInt(const std::string& str, int& result)
{
    char* endptr;
    long i = strtol(str.c_str(),&endptr,10);
    bool ok = true;
    while (char c = *endptr++) 
        if(!isspace(c)) 
            ok = false;

    if(ok)
        result = i;

    return ok;
}


/*
 *  Parse a double.
 *  If valid, return true and pass result via argument 'result'.
 */

bool StringFns::parseDouble(const std::string& str, double& result)
{
    char* endptr;
    double d = strtod(str.c_str(),&endptr);
    bool ok = true;
    while (char c = *endptr++) 
        if(!isspace(c)) 
            ok = false;

    if(ok)
        result = d;

    return ok;
}
